package org.jtheque.films.stats.services.impl.utils;

/*
 * This file is part of JTheque.
 *
 * JTheque is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License.
 *
 * JTheque is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with JTheque.  If not, see <http://www.gnu.org/licenses/>.
 */

import org.jfree.data.general.DefaultPieDataset;

import org.jtheque.utils.count.Counter;
import org.jtheque.utils.count.Counters;
import org.jtheque.films.persistence.od.FilmImpl;
import org.jtheque.films.services.able.IActorService;
import org.jtheque.films.services.able.IFilmsService;
import org.jtheque.films.services.able.IRealizersService;
import org.jtheque.primary.od.abstraction.NotablePerson;
import org.jtheque.primary.od.KindImpl;

import javax.annotation.Resource;
import java.util.HashMap;
import java.util.Map;
import java.util.List;

/**
 * This manage enable us to generate statistics. This class is a singleton.
 *
 * @author Baptiste Wicht
 */
public final class StatsCalculator implements StatsInformations {
    private static final StatsInformations INSTANCE = new StatsCalculator();

    @Resource
    private IFilmsService filmsService;

    @Resource
    private IActorService actorService;

    @Resource
    private IRealizersService realizersService;

    /* Statistic's datas */


    private final Map<String, DefaultPieDataset> sets = new HashMap<String, DefaultPieDataset>(7);
    private final Map<String, Counters> data = new HashMap<String, Counters>(8);

    static final String GENERALS = "generals";

    /**
     * Private constructor, create a new <code>Statistics</code>.
     */
    private StatsCalculator() {
        super();
        
        data.put(GENERALS, new Counters());
        data.get(GENERALS).addCounter("films");
        data.get(GENERALS).addCounter("actors");
        data.get(GENERALS).addCounter("realizers");
        data.get(GENERALS).addCounter("duration");

        sets.put("films-kinds", new DefaultPieDataset());
        sets.put("films-types", new DefaultPieDataset());
        sets.put("films-notes", new DefaultPieDataset());
        sets.put("realizers-notes", new DefaultPieDataset());
        sets.put("realizers-countries", new DefaultPieDataset());
        sets.put("actors-notes", new DefaultPieDataset());
        sets.put("actors-countries", new DefaultPieDataset());
    }

    /**
     * Return the unique instance of <code>Statistics</code>.
     *
     * @return The instance of the class
     */
    public static StatsInformations getInstance() {
        return INSTANCE;
    }

    @Override
    public void generateStats() {
        CounterUtils.clearButGenerals(data);

        extractStatisticsOfFilms(data);
        extractStatisticsOfActors(data);
        extractStatisticsOfRealizers(data);

        populateDataset(sets.get("films-kinds"), "kinds", data);
        populateDataset(sets.get("films-types"), "types", data);
        populateDataset(sets.get("films-notes"), "f-notes", data);
        populateDataset(sets.get("actors-countries"), "a-countries", data);
        populateDataset(sets.get("actors-notes"), "a-notes", data);
        populateDataset(sets.get("realizers-countries"), "r-countries", data);
        populateDataset(sets.get("realizers-notes"), "r-notes", data);
    }

    /**
     * Extract the statistics of the films.
     *
     * @param data The data of stats.
     */
    private void extractStatisticsOfFilms(Map<String, Counters> data) {
        data.put("kinds", new Counters());
        data.put("types", new Counters());
        data.put("f-notes", new Counters());
        data.get(GENERALS).getCounter("films").clear();
        data.get(GENERALS).getCounter("duration").clear();

        for (FilmImpl film : filmsService.getFilms()) {
            data.get(GENERALS).getCounter("duration").add(film.getDuration());
            data.get(GENERALS).getCounter("films").increment();

            if (film.hasKinds()) {
                for (KindImpl kind : film.getKinds()) {
                    data.get("kinds").getCounterOrAdd(kind.getAffichableText()).increment();
                }
            }

            if (film.hasType()) {
                data.get("types").getCounterOrAdd(film.getTheType().getAffichableText()).increment();
            }

            if (film.getNote() != null) {
                data.get("f-notes").getCounterOrAdd(film.getNote().getInternationalizedText()).increment();
            }
        }
    }

    /**
     * Extract the statistics of the realizers.
     *
     * @param data The data of stats.
     */
    private void extractStatisticsOfRealizers(Map<String, Counters> data) {
        extractStatisticsPersons(data, "realizers", 'r');
    }

    /**
     * Extract the statistics of the actors.
     *
     * @param data The data of stats.
     */
    private void extractStatisticsOfActors(Map<String, Counters> data) {
        extractStatisticsPersons(data, "actors", 'a');
    }

    /**
     * Extract the statistics of the persons.
     *
     * @param data The data of stats.
     * @param name The name of counter.
     * @param c The prefix for intern counters. 
     */
    private void extractStatisticsPersons(Map<String, Counters> data, String name, char c) {
        data.put(c + "-countries", new Counters());
        data.put(c + "-notes", new Counters());
        data.get(GENERALS).getCounter(name).clear();

        List<? extends NotablePerson> persons;

        if("actors".equals(name)){
            persons = actorService.getActors();
        } else {
            persons = realizersService.getRealizers();
        }

        for (NotablePerson r : persons) {
            data.get(GENERALS).getCounter(name).increment();

            if(r.getTheCountry() != null){
                data.get(c + "-countries").getCounterOrAdd(r.getTheCountry().getAffichableText()).increment();
            }

            if(r.getNote() != null){
                data.get(c + "-notes").getCounterOrAdd(r.getNote().getInternationalizedText()).increment();
            }
        }
    }

    /**
     * Populate a dataset with the contents of the 2 lists.
     *
     * @param dataset The dataset to populate.
     * @param name    The name of the map.
     * @param datas   The datas.
     */
    private static void populateDataset(DefaultPieDataset dataset, String name, Map<String, Counters> datas) {
        for (Map.Entry<String, Counter> counter : datas.get(name)) {
            dataset.setValue(counter.getKey(), counter.getValue().getValue());
        }
    }

    @Override
    public int getNumberOfFilms() {
        return data.get(GENERALS).getCounter("films").getValue();
    }

    @Override
    public int getNumberOfActors() {
        return data.get(GENERALS).getCounter("actors").getValue();
    }

    @Override
    public int getNumberOfRealizers() {
        return data.get(GENERALS).getCounter("realizers").getValue();
    }

    @Override
    public int getTotalDurationOfFilms() {
        return data.get(GENERALS).getCounter("duration").getValue();
    }

    @Override
    public DefaultPieDataset getStatsOfFilmByKind() {
        return sets.get("films-kinds");
    }

    @Override
    public DefaultPieDataset getStatsOfFilmByType() {
        return sets.get("films-types");
    }

    @Override
    public DefaultPieDataset getStatsOfFilmByNote() {
        return sets.get("films-notes");
    }

    @Override
    public DefaultPieDataset getStatsOfActorsByCountry() {
        return sets.get("actors-countries");
    }

    @Override
    public DefaultPieDataset getStatsOfActorsByNote() {
        return sets.get("actors-notes");
    }

    @Override
    public DefaultPieDataset getStatsOfRealizersByCountry() {
        return sets.get("realizers-countries");
    }

    @Override
    public DefaultPieDataset getStatsOfRealizersByNote() {
        return sets.get("realizers-notes");
    }
}